@@ -11,9 +11,11 @@ from gcmpy.tools.joint_excess_joint_degree_matrices import (
 )
 from gcmpy.tools.proposal_edge import ProposalEdge
 from gcmpy.tools.markov_chain_monte_carlo import MarkovChainMonteCarlo
+from gcmpy.tools.joint_excess_joint_degree_keys_view import JointExcessJointDegreeKeysView
 
 
-class ErrorMarkovChainMonteCarloRewiring(Exception): ...
+class ErrorMarkovChainMonteCarloRewiring(Exception):
+    ...
 
 
 class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
@@ -22,14 +24,16 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
     Stochastically rewires `_network` so that the target correlations match `ejks`.
     """
 
-    _network: Network = None
-    _convergence_limit: int = None
-    _search_limit: int = 25
-    _ejks: JointExcessJointDegreeMatrices = None
-    _logger: Logger = Logger("MarkovChainMonteCarloRewiring")
-    _proposal_edges: list[ProposalEdge] = []
-
     def __init__(self, params: dict):
+        self._network: Network = None
+        self._convergence_limit: int = None
+        self._search_limit: int = 25
+        self._ejks: JointExcessJointDegreeMatrices = None
+        self._logger: Logger = Logger("MarkovChainMonteCarloRewiring")
+        self._proposal_edges: list[ProposalEdge] = []
+        self._proposal_count: int = 0
+        self._proposals_accepted: int = 0
+        self._acceptance_ratio: list = []
         try:
             self._network: Network = params[ToolsNames.NETWORK]
             self._ejks: JointExcessJointDegreeMatrices = params[ToolsNames.EJKS]
@@ -37,7 +41,7 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
                 self._convergence_limit: int = params[ToolsNames.CONVERGENCE_LIMIT]
             else:
                 # set a limit for the number of swaps before termination
-                self._convergence_limit = 10 * self._network._G.edges()
+                self._convergence_limit = 10 * self._network.G.edges()
             if ToolsNames.SEARCH_LIMIT in params:
                 self._search_limit: int = params[ToolsNames.SEARCH_LIMIT]
         except Exception as e:
@@ -173,7 +177,7 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
 
     def get_swapped_joint_excess_degree_key(
         self, G: nx.Graph, e0: tuple, e1: tuple, u0: int, v0: int, index: int
-    ) -> tuple:
+    ) -> JointExcessJointDegreeKeysView:
         """
         Returns the swapped joint excess degree key tuple for two edges e0, e1 in network G.
         :param G: nx.Graph
@@ -182,7 +186,7 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
         :param u0: left vertex
         :param v0: right vertex
         :param index: index in joint degree of edge topology
-        :returns key pair: joint excess joint degree key tuples for (u0,v1) and (v0,u1)
+        :returns JointExcessJointDegreeKeysView: object to view excess joint degree tuples
         """
         u1: int = self.get_other_vertex(u0, e0)
         v1: int = self.get_other_vertex(v0, e1)
@@ -196,7 +200,8 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
         for jd in us_jd:
             jd[index] -= 1
             us_excess_jd.append(tuple(jd))
-        return us_excess_jd[0] + us_excess_jd[3], us_excess_jd[2] + us_excess_jd[1]
+
+        return JointExcessJointDegreeKeysView(us_excess_jd)
 
     def append_proposal_edges(
         self, G: nx.Graph, u0: int, old_edge: tuple, new_edge:  tuple
@@ -223,8 +228,8 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
         Evaluates the Metropolis condition for the algorithm.
 
         .. math::
-            \pi=\prod_{\nu}\prod_i\frac{e_{\tau,k_{u_1},\nu,k_{v_i}}e_{\tau,k_{v_1},
-            \nu,k_{v_i}}}{e_{\tau,k_{u_1},\nu,k_{u_i}}e_{\tau,k_{v_1},\nu,k_{v_i}}}
+            \\pi=prod_{\\nu}\\prod_i \\frac{e_{tau,k_{u_1},\\nu,k_{v_i}}e_{\tau,k_{v_1},
+            nu,k_{v_i}}}{e_{\tau,k_{u_1},\nu,k_{u_i}}e_{\tau,k_{v_1},\nu,k_{v_i}}}
 
         Ensures that corresponding edges are swapped by creating a topology hashmap
         of (v0,v1) edges so that subgraphs with different orbits swap correct edges.
@@ -266,13 +271,25 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
                 G, v0, e1, (v0, self.get_other_vertex(u0, e0))
             )
 
-            # these are the excess degree tuples of the proposed edges
-            key1, key2 = self.get_swapped_joint_excess_degree_key(
+            # create an excess degree key view object
+            key_view: JointExcessJointDegreeKeysView = self.get_swapped_joint_excess_degree_key(
                 G, e0, e1, u0, v0, index
             )
 
+            u0v1_key: tuple = key_view.get_u0v1()
+            v0u1_key: tuple = key_view.get_v0u1()
+            u0u1: tuple = key_view.get_u0u1()
+            u1u0: tuple = key_view.get_u1u0()
+            v0v1: tuple = key_view.get_v0v1()
+            v1v0: tuple = key_view.get_v1v0()
+
+            # test if swapped keys change anything, if not, silent return False.
+            starting_keys: set = {u0u1, u1u0, v0v1, v1v0}
+            if set([u0v1_key, v0u1_key]).issubset(starting_keys):
+                return False
+
             try:
-                top *= self._ejks.ejks[topology][key1] * self._ejks.ejks[topology][key2]
+                top *= self._ejks.ejks[topology][u0v1_key] * self._ejks.ejks[topology][v0u1_key]
             except KeyError as e:
                 # guard due to GCM algorithm honouring the handshaking lemma by inserting
                 # additional motif. Ensure this is not statistically significant, unlikely
@@ -321,12 +338,12 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
 
         return False
 
-    def rewire(self) -> None:
+    def rewire(self) -> nx.Graph:
         """
         Entrypoint to MCMC rewiring algorithm
         """
         # pull a copy of the network, careful still has edge data?
-        G: nx.Graph = self._network.G
+        G: nx.Graph = self._network.G.copy()
 
         number_of_edges: int = G.number_of_edges()
 
@@ -339,6 +356,11 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
         convergence_count: int = 0
         while convergence_count <= self._convergence_limit:
 
+            if convergence_count % 100 == 0 and self._proposal_count != 0:
+                self._acceptance_ratio.append(
+                    float(self._proposals_accepted)/float(self._proposal_count)
+                )
+
             # pick an edge at random.
             e0: tuple = EdgeSet.draw()
             u0: int = e0[0]
@@ -405,7 +427,7 @@ class MarkovChainMonteCarloRewiring(MarkovChainMonteCarlo):
                         by swap {number_of_edges} vs {G.number_of_edges()}"
                     )
 
-        self._network.G = G
+        return G
 
     @property
     def network(self) -> Network:
